#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <locale.h>
#include <math.h>
#include <dirent.h>
#include <sys/types.h>

void createCsv( char* inputFile, char* outputFile );

void scanDirectory( char* dirName );

void printHelp();


int main( int argc, char **argv ) 
{
    if( argc < 2 )
    {
        printf( "No input file. Aborting... \n" );
        return 0;
    }

    setlocale( LC_ALL, "" );
    setlocale( LC_ALL, NULL );
    int i;
    for( i = 0; i < argc; i = i + 1 )
    {
        if( strcmp( argv[i], "-h" ) == 0 ||
            strcmp( argv[i], "-help" ) == 0 ||
            strcmp( argv[i], "--h" ) == 0 ||
            strcmp( argv[i], "--help" ) == 0 )
        {
            printHelp();
            return 0;
        }

        if( strcmp( argv[i], "-d" ) == 0 && i < argc - 1 )
        {
            scanDirectory( argv[i+1] );
            return 0;
        }

        if( strcmp( argv[i], "-dc" ) == 0 )
        {
            char curDir[1024];
            memset( curDir, 0, 1024 );
            getcwd( curDir, sizeof( curDir ) );
            scanDirectory( curDir );
            return 0;
        }
        if( strcmp( argv[i], "-f" ) == 0 && i < argc - 1 )
        {
            char fileName[1024];
            memset( fileName, 0, 1024 );
            strcpy( fileName, argv[i + 1] );
            strcat( fileName , ".csv" );
            createCsv( argv[i + 1], fileName );
            return 0;
        }
    }

    if( argc == 2 )
    {
        char fileName[1024];
        memset( fileName, 0, 1024 );
        strcpy( fileName, argv[1] );
        strcat( fileName, ".csv");
        createCsv( argv[1], fileName );
    }
    return 0;
}

void createCsv( char* inputFile, char* outputFile )
{
    FILE* binaryFile;
    FILE* csvFile;

    char valStr[20];
    char timeStr[20];

    binaryFile = fopen( inputFile, "rb" );
    if( binaryFile == NULL )
    {
        return;
    }
    csvFile = fopen( outputFile, "w" );
    if( csvFile == NULL )
    {
        return;
    }

    long lastStamp = 0;
    long timeStamp = 0;
    double value = 0;
    long tm1000 = 0;
    long msecs = 0;
    do
    {
        timeStamp = 0;
        value = 0;
        memset( valStr, 0, 20 );
        memset( timeStr, 0, 20 );
        fread( (char*)&timeStamp, sizeof( long ), 1, binaryFile );

        if( timeStamp <= 0 )
        {
            break;
        }

        time_t time_val = ( timeStamp - 62135596800000 ) / 1000;

        tm1000 = ( timeStamp / 1000 ) * 1000;
        msecs =  ( timeStamp - tm1000 );

        fread( (char*)&value, 8, 1, binaryFile );

        if( isinf( value ) )
        {
            sprintf( valStr, "%s", "∞" );
        }
        else
        {
            sprintf( valStr, "%f", value );
        }

        if( lastStamp == 0 )
        {
            lastStamp = timeStamp;
        }

        long diff = timeStamp - lastStamp;

        strftime( timeStr, 20, "%d-%m-%Y %H:%M:%S", gmtime( &time_val ) );
        fprintf( csvFile, "%s.%i;%i;%s \n", timeStr, msecs, diff, valStr );

    }while( !feof( binaryFile ) );

    fclose( binaryFile );
    fclose( csvFile );
}

void scanDirectory( char* dirName )
{
    DIR* curDirectory = 0;
    struct dirent *entry = 0;

    curDirectory = opendir( dirName );
    entry = readdir( curDirectory );
    while( entry != NULL )
    {
        if( strstr( entry->d_name, "track0" ) != NULL
            && strstr( entry->d_name, ".csv" ) == NULL )
        {
            char fileName[1024];
            memset( fileName, 0, 1024 );
            strcpy( fileName, dirName );
            strcat( fileName, "/" );
            strcat( fileName, entry->d_name );
            strcat( fileName, ".csv" );
            createCsv( entry->d_name, fileName );
        }
        entry = readdir( curDirectory );
    }
    closedir( curDirectory );
}

void printHelp()
{
    printf( "-h: show this help\n" );
    printf( "-f file_name - parse single file\n" );
    printf( "-d /directory_name - parse all files in directory\n" );
    printf( "-dc - parse all files in current directory\n" );
}
